/*
 *		M U L T I - D E V I C E   Q U E U E I N G   S Y S T E M
 *
 *				  Q U E U E . H
 *
 *				Michael John Muuss
 *			      Douglas P. Kingston III
 *
 *				Revision History
 *				----------------
 *	30 Mar 83  DPK	Added failed open count to device structure.
 *			This change courtesy of Dave Wilson at Boeing.
 */

#include	<sys/types.h>
#include	"qconf.h"

#define FORMSIZ		8	/* #chars to a forms name */
#define	QDIRSIZ		14	/* #chars for internal filenames */
#define	QUESIZ		QDIRSIZ	/* #chars in a queue name */
#define	PATHSIZ		128	/* Largest pathname we will support */
#define	LINESIZ		134	/* Largest line we will support */
#define	MAXFIELDS	20	/* Largest number or parsable fields */


/*
 *	Request Structure
 */
struct request {
	char	r_name[QDIRSIZ];
	char	r_que[QUESIZ];
	int	r_pri;
	int	r_status;
	int	r_uid;
	int	r_gid;
	char	r_forms[FORMSIZ];
	int	r_options;
	int	r_copies;
	long	r_size;
	time_t	r_tstart;
	time_t	r_tsubmit;
	time_t	r_tdelay;
	struct request *r_nextreq;
};

/* r_status flags */
#define	SKILL	000040			/* Kill this request */
#define	SHOLD	000020			/* Do not start this task */
#define	SDELAY	000010			/* Delayed by daemon */
#define SRUN	000001			/* Job is being processed */

/* r_options flags */
#define	O_MAIL	000001		/* Send mail upon request completion */

/*
 *	Individual Queue Information
 */
struct queue {
	char	*q_name;		/* Logical queue name */
	int	q_status;		/* Status bits for the queue */
	struct request *q_readyreq;	/* Sorted list of requests, hd first */
	struct queue *q_nextq;		/* next in complete list of queues */
};

/* Definition of q_status bits */
#define	Q_TOUCH		0100000
#define	Q_UPDATE	0040000

/*
 *	Device Structure
 */
struct device {
	char	*dv_dname;		/* Shorthand name (lp0) */
	char	*dv_device;		/* Formal name (/dev/lp0) */
	char	dv_forms[FORMSIZ];	/* Current forms in device */
	int	dv_status;		/* Status bits for this device */
	int	dv_failed;		/* Number of failed open attempts */
	int	dv_pid;			/* PID of servicing filecontrol() */
	struct request *dv_actreq;	/* Request currently being served */
	struct queue *dv_actque;	/* Queue from which dv_actreq came */
	char	*dv_server;		/* Server to be used for request */
	struct device *dv_nextdev;	/* Next in a complete list of devs */
	struct qmap *dv_maphd;		/* Head of queue mappings for dev */
	time_t	dv_mtime;		/* Last update of status file */
};

/* Definition of dv_status bits */
#define	D_DISABLED	0100000		/* Disables device from further use */
#define	D_RROBIN	0000040
#define	D_TOUCH		0000020		/* For housekeeping during rdconf() */
#define	D_FAILED	0000010		/* Disables device from further use */
#define	D_ANYFORM	0000004		/* Any form can be queued to device */
#define	D_SKIPMSG	0000002		/* Dont bother sending notification */
#define	D_RESET		0000001		/* Device characteristics changed */

#define	D_DAEBITS	0000077		/* Above bits "owned" by daemon */


/*
 *	Queue/Device Mapping Structure
 */
struct qmap {
	struct queue *m_q;
	char *m_server;		/* The shell line to exec server */
	struct qmap *m_nextmap;
};

/*
 *	Format of binary data in qlist files
 *	Updated by Daemon, read by Q(I).
 */
struct qdata {
	int	qd_uid;
	int	qd_seqno;
};

/*
 *	File name format (character offsets)
 */
#define	QN_Q	0
#define	QN_UID	1
#define	QN_SEQ	7

/*
 *	Structure of return from wait() system call
 *	Note:  This is probably machine dependent
 */
struct status {
	char	pstat;	/* Process exit status */
	char	ret;	/* Return code from process */
};

/*
 * Open(II) access modes
 */
#define READ	0	/* Normal access */
#define WRITE	1
#define RW	2

/*
 *  Access(II) modes
 */
#define	RACCESS	04		/* Read access */
#define	WACCESS	02		/* Write access */
#define	XACCESS	01		/* Execute access */
#define	EXISTS	0		/* Does it exist at all? */

struct lockdata {
	int	lfd;		/* File descriptor for a lock */
	char	*lockfile;	/* Name of the lock file */
};

/*
 *	Stuff used for configuring daemon and enqueuers
 */
#define	INTVAR	1
#define	STRPVAR	2

struct paramtab {
	char	*paramstr;	/* Name of variable */
	int	vartype;
	char	**parmp;	/* Address of parameter to be set, or NULL */
	int	(*setfunc)();	/* Function to be called with parameter */
};


typedef	struct lockdata	lock_t;

/* Common external definitions */
extern	int	errno;
extern	char	*sys_errlist[];		/* Perror strings are here */

#ifndef NULL
#define NULL	(char *)0		/* in case stdio.h is not used */
#endif

#define	SAME	0		/* what strcmp() returns for match */
#define	NOWAIT	-1		/* passed to lock() to force one try only */

#define	NOISY	1		/* complain if error occurs */
#define	QUIET	0		/* don't complain if error occurs */

#ifdef BSD41
#define	signal	sigset		/* use uninterruptalbe signal */
#endif

/* Common device specific options, formerly in print.h */
#define	NOCOPY		000001
#define	NOFF		000002
#define	NOINDENT	000004
#define	NOKEEP		000010
