#
/*
 *	    M U L T I - D E V I C E   Q U E U E I N G   S Y S T E M
 *
 *			   Douglas P. Kingston III
 *
 *		     Versatec PLOT Server Process  (VPLOTSERVER)
 *
 *				Revision History
 *				================
 */
#include <errno.h>
#include <stdio.h>
#include <pwd.h>
#include <sgtty.h>
#include <signal.h>
#include "vp.h"
#include "queue.h"
#include "qreplys.h"

#define	VERSATEC
/* Accounting assumes roll paper */
#define ACCTFILE	"/usr/adm/vpdacct"

struct	passwd *pw;
time_t	timenow;
struct	request r;
int	lpoptions;
long	linecount = 0L;	/* number of lines printed by this server */
char	source[LINESIZ]; /* source of request, account name or net address */
char	obuf[BUFSIZ];	/* remember the output goes to stdout */
int	numcols = 132;
int	numhdrs = 1;
int	numtrailers = 0;
char	*title = 0;
char	*arg0;

struct	request *gethdr();
struct	passwd *getpwuid();
char	*index();
char	*ctime();
char	*strdup();
int	doflush();
int	dorestart();
long	vplotn();
		/* vplotn() returns the number of raster lines output */
extern int	vprtmode[];
extern int	vpltmode[];

extern	char *Sitefile;
extern	char *Qdatadir;

#define	FF	014	/* A formfeed character */

/*
 *		M A I N  ( )
 *  As a MDQS queue server, this process will be started up
 *  with the queue control file as the standard input, and
 *  the device of interest on the standard output.
 */
main (argc, argv)
int	argc;
char	**argv;
{
	char	nfiles = 0;
	char	datafile[LINESIZ];
	char	linebuf[LINESIZ];
	char	*cp;
	
	signal( SIGHUP, SIG_IGN );		/* Paranoid. */
	signal( SIGEMT, dorestart );
	signal( SIGTERM, doflush );

	arginit(argc, argv);
	time( &timenow );
	setbuf( stdout, obuf );
	ioctl( fileno( stdout ), SETSTATE, vprtmode );

	if (gethdr (stdin, &r) == (struct request *)NULL) {
		err("Error reading request\n");
		retmsg (RP_FATAL, "Error reading request header");
		exit (10);
	}

	/* get password info for user request */
	pw = getpwuid (r.r_uid);

	while (fgets (linebuf, sizeof (linebuf), stdin) != NULL) {
		if ((cp = index (linebuf, '\n')) != 0)
			*cp = 0;
		switch (linebuf[0]) {
		case 'B':
			sprintf (datafile, "%s/%s", Qdatadir, &linebuf[1]);
			prheader (datafile, &r);
			unlink (datafile);
			break;

		case 'D':
			if( chdir( &linebuf[1] ) < 0 ) {
				printf("\n\nError trying chdir (%s,%s)\n%c",
				&linebuf[1], sys_errlist[errno], FF );
				chdir( "/tmp" );	/* A safe default */
			}
			break;

		case 'U':
			strcpy( source, &linebuf[1] );
			break;

		case 'O':
			setoptions (&linebuf[1]);
			break;

		case 'H':
			prheader (&linebuf[1], &r);
			break;

		case 'I':
			sprintf (datafile, "%s/%s", Qdatadir, &linebuf[1]);
			plot (datafile);
			unlink (datafile);
			break;

		case 'F':
			if (plot (&linebuf[1]) < 0)
					break;
			if (lpoptions & NOKEEP)
				unlink (&linebuf[1]);
			break;

		case 'T':
			title = strdup(&linebuf[1]);
			break;

		default:
			break;
		}
	}
	if(numtrailers)
		prtrailer(&r);

	account();
	retmsg (RP_OK, 0);
	exit (0);
}

/*
 *		S E T O P T I O N S  ( )
 *  Setoptions is passed the ascii string numerical representation
 *  of the lpoptions flags word.  The bit definitions are in queue.h.
 *  The string is a decimal representation.
 */
setoptions (optstr)
char *optstr;
{
	lpoptions = atoi (optstr);

	/* Should do something with NOFF (no formfeeds) here. (stty) */
}

/*
 *		P L O T ( )
 *  Plot a raster file onto the versatec.
 */
plot( file )
char *file;
{
	FILE	*df;
	int	cnt;

	if ((df = fopen (file, "r")) == NULL) {
		printf ("\n\n***** Could not open input file (%s, %s) *****\n",
			file, sys_errlist[errno] );
		putchar( FF );
		return( -1 );
	}
	for( cnt = r.r_copies; cnt > 0; cnt--) {
		linecount += vplotn (df, r.r_size - linecount);
		if( cnt ) {
			rewind( df );
		}
	}
	fclose( df );
	if (r.r_size > 0 && linecount > r.r_size)
		printf ("\n----- Printing limit of %ld lines exceeded, printing aborted. -----\n", r.r_size);
	putchar( FF );
	return( 0 );
}

/*
 *	A R G I N I T
 *
 *	Parse args of lineprinter like servers
 */
arginit(argc, argv)
int	argc;
char	**argv;
{
	arg0 = argv[0];

	while(*++argv) {
		/*
		 *  Number of columns argument
		 */
		if (strncmp(*argv, "columns=", 8) == SAME) {
			numcols = atoi(*argv+8);
			if( numcols < 1 ) {
				numcols = 132;
				err("bad argument '%s'", *argv);
			}
			continue;
		}
		/*
		 *  Number of headers argument
		 */
		if (strncmp(*argv, "nheaders=", 9) == SAME) {
			numhdrs = atoi(*argv+9);
			if( numhdrs < 0 ) {
				numhdrs = 1;
				err("bad argument '%s'", *argv);
			}
			continue;
		}
		/*
		 *  Number of trailers argument
		 */
		if (strncmp(*argv, "ntrailers=", 10) == SAME) {
			numtrailers = atoi(*argv+10);
			if( numtrailers < 0 ) {
				numtrailers = 0;
				err("bad argument '%s'", *argv);
			}
			continue;
		}
	}
}

/*
 *		E R R ( )
 *	Called by banner() and its subfunctions, amongst others
 */
/* VARARGS */
err( fmt, a, b, c, d )
char *fmt;
{
	fputs( "VPLOTSERVER: ", stderr );
	fprintf( stderr, fmt, a, b, c, d );
	putc('\n', stderr);
}

account()
{
	register int a;
	char buf[80];

	a = open(ACCTFILE, 1);
	if (a < 0)
		return;
	lseek(a, 0L, 2);
	sprintf(buf, "t%6.2f\t%s\n", (linecount / 25.0) / 12.0, pw->pw_name);
	write(a, buf, strlen(buf));
	close(a);
}
